'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _GitTree = require('../models/GitTree');

var _GitTree2 = _interopRequireDefault(_GitTree);

var _GitCommit = require('../models/GitCommit');

var _GitCommit2 = _interopRequireDefault(_GitCommit);

var _GitObjectManager = require('../managers/GitObjectManager');

var _GitObjectManager2 = _interopRequireDefault(_GitObjectManager);

var _GitIndexManager = require('../managers/GitIndexManager');

var _GitIndexManager2 = _interopRequireDefault(_GitIndexManager);

var _resolveRef = require('../utils/resolveRef');

var _resolveRef2 = _interopRequireDefault(_resolveRef);

var _flatFileListToDirectoryStructure = require('../utils/flatFileListToDirectoryStructure');

var _flatFileListToDirectoryStructure2 = _interopRequireDefault(_flatFileListToDirectoryStructure);

var _write = require('../utils/write');

var _write2 = _interopRequireDefault(_write);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function constructTree({ gitdir, inode }) /*: string */{
  // use depth first traversal
  let children = inode.children;
  for (let inode of children) {
    if (inode.type === 'tree') {
      inode.metadata.mode = '040000';
      inode.metadata.oid = await constructTree({ gitdir, inode });
    }
  }
  let entries = children.map(inode => ({
    mode: inode.metadata.mode,
    path: inode.basename,
    oid: inode.metadata.oid,
    type: inode.type
  }));
  const tree = _GitTree2.default.from(entries);
  let oid = await _GitObjectManager2.default.write({
    gitdir,
    type: 'tree',
    object: tree.toObject()
  });
  return oid;
}

exports.default = async function commit({
  gitdir,
  author,
  committer,
  message,
  privateKeys
}) {
  // Fill in missing arguments with default values
  committer = committer || author;
  let authorDateTime = author.date || new Date();
  let committerDateTime = committer.date || authorDateTime;
  let oid;
  await _GitIndexManager2.default.acquire(`${gitdir}/index`, async function (index) {
    const inode = (0, _flatFileListToDirectoryStructure2.default)(index.entries);
    const treeRef = await constructTree({ gitdir, inode });
    const parent = await (0, _resolveRef2.default)({ gitdir, ref: 'HEAD' });
    let comm = _GitCommit2.default.from({
      tree: treeRef,
      parent: [parent],
      author: {
        name: author.name,
        email: author.email,
        timestamp: author.timestamp || Math.floor(authorDateTime.valueOf() / 1000),
        timezoneOffset: author.timezoneOffset || 0
      },
      committer: {
        name: committer.name,
        email: committer.email,
        timestamp: committer.timestamp || Math.floor(committerDateTime.valueOf() / 1000),
        timezoneOffset: committer.timezoneOffset || 0
      },
      message
    });
    if (privateKeys) {
      comm = await comm.sign(privateKeys);
    }
    oid = await _GitObjectManager2.default.write({
      gitdir,
      type: 'commit',
      object: comm.toObject()
    });
    // Update branch pointer
    const branch = await (0, _resolveRef2.default)({ gitdir, ref: 'HEAD', depth: 2 });
    await (0, _write2.default)(_path2.default.join(gitdir, branch), oid + '\n');
  });
  return oid;
};
//# sourceMappingURL=commit.js.map