'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Git = undefined;
exports.default = git;

var _githubUrlToObject = require('github-url-to-object');

var _githubUrlToObject2 = _interopRequireDefault(_githubUrlToObject);

var _init = require('./commands/init.js');

var _init2 = _interopRequireDefault(_init);

var _fetch = require('./commands/fetch.js');

var _fetch2 = _interopRequireDefault(_fetch);

var _checkout = require('./commands/checkout.js');

var _checkout2 = _interopRequireDefault(_checkout);

var _list = require('./commands/list.js');

var _list2 = _interopRequireDefault(_list);

var _add = require('./commands/add.js');

var _add2 = _interopRequireDefault(_add);

var _remove = require('./commands/remove.js');

var _remove2 = _interopRequireDefault(_remove);

var _commit = require('./commands/commit.js');

var _commit2 = _interopRequireDefault(_commit);

var _verify = require('./commands/verify.js');

var _verify2 = _interopRequireDefault(_verify);

var _packObjects = require('./commands/pack-objects.js');

var _packObjects2 = _interopRequireDefault(_packObjects);

var _push = require('./commands/push.js');

var _push2 = _interopRequireDefault(_push);

var _getConfig = require('./commands/getConfig.js');

var _getConfig2 = _interopRequireDefault(_getConfig);

var _setConfig = require('./commands/setConfig.js');

var _setConfig2 = _interopRequireDefault(_setConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Class is merely a fluent command/query builder
function git(dir) {
  return new Git(dir);
}

class Git {
  constructor(dir) {
    if (dir) {
      this.workdir = dir;
      this.gitdir = `${dir}/.git`;
    }
    this.operateRemote = 'origin';
  }
  workdir(dir) {
    this.workdir = dir;
    return this;
  }
  gitdir(dir) {
    this.gitdir = dir;
    return this;
  }
  githubToken(token) {
    this.operateToken = token;
    return this;
  }
  remote(name) {
    this.operateRemote = name;
    return this;
  }
  author(name) {
    this.operateAuthorName = name;
    return this;
  }
  email(email) {
    this.operateAuthorEmail = email;
    return this;
  }
  datetime(date) {
    this.operateAuthorDateTime = date;
    return this;
  }
  timestamp(seconds) {
    // seconds since unix epoch
    this.operateAuthorTimestamp = seconds;
    return this;
  }
  signingKey(asciiarmor) {
    this.privateKeys = asciiarmor;
    return this;
  }
  verificationKey(asciiarmor) {
    this.publicKeys = asciiarmor;
    return this;
  }
  outputStream(stream) {
    this.outputStream = stream;
    return this;
  }
  async init() {
    await (0, _init2.default)(this.gitdir);
  }
  async fetch(url) {
    await (0, _fetch2.default)({
      gitdir: this.gitdir,
      user: (0, _githubUrlToObject2.default)(url).user,
      repo: (0, _githubUrlToObject2.default)(url).repo,
      ref: (0, _githubUrlToObject2.default)(url).branch,
      remote: this.operateRemote,
      token: this.operateToken
    });
  }
  async checkout(ref) {
    await (0, _checkout2.default)({
      workdir: this.workdir,
      gitdir: this.gitdir,
      ref,
      remote: this.operateRemote
    });
  }
  async clone(url) {
    await (0, _init2.default)(this.gitdir);
    await (0, _fetch2.default)({
      gitdir: this.gitdir,
      user: (0, _githubUrlToObject2.default)(url).user,
      repo: (0, _githubUrlToObject2.default)(url).repo,
      ref: (0, _githubUrlToObject2.default)(url).branch,
      remote: this.operateRemote,
      token: this.operateToken
    });
    await (0, _checkout2.default)({
      workdir: this.workdir,
      gitdir: this.gitdir,
      ref: (0, _githubUrlToObject2.default)(url).branch,
      remote: this.operateRemote
    });
  }
  async list() {
    return (0, _list2.default)({
      gitdir: this.gitdir
    });
  }
  async add(filepath) {
    return (0, _add2.default)({
      gitdir: this.gitdir,
      workdir: this.workdir,
      filepath
    });
  }
  async remove(filepath) {
    return (0, _remove2.default)({
      gitdir: this.gitdir,
      filepath
    });
  }
  async commit(message) {
    return (0, _commit2.default)({
      gitdir: this.gitdir,
      author: {
        name: this.operateAuthorName || (await this.getConfig('user.name')),
        email: this.operateAuthorEmail || (await this.getConfig('user.email')),
        timestamp: this.operateAuthorTimestamp,
        date: this.operateAuthorDateTime
      },
      committer: {
        name: this.operateAuthorName || (await this.getConfig('user.name')),
        email: this.operateAuthorEmail || (await this.getConfig('user.email')),
        timestamp: this.operateAuthorTimestamp,
        date: this.operateAuthorDateTime
      },
      message,
      privateKeys: this.privateKeys
    });
  }
  async verify(ref) {
    return (0, _verify2.default)({
      gitdir: this.gitdir,
      publicKeys: this.publicKeys,
      ref
    });
  }
  async pack(oids) {
    return (0, _packObjects2.default)({
      gitdir: this.gitdir,
      outputStream: this.outputStream,
      oids
    });
  }
  async push(ref) {
    let url = await (0, _getConfig2.default)({
      gitdir: this.gitdir,
      path: `remote "${this.operateRemote}".url`
    });
    console.log('url =', url);
    return (0, _push2.default)({
      gitdir: this.gitdir,
      ref,
      url,
      auth: {
        username: this.operateToken,
        password: this.operateToken
      }
    });
  }
  async getConfig(path) {
    return (0, _getConfig2.default)({
      gitdir: this.gitdir,
      path
    });
  }
  async setConfig(path, value) {
    return (0, _setConfig2.default)({
      gitdir: this.gitdir,
      path,
      value
    });
  }
}
exports.Git = Git;
//# sourceMappingURL=index.js.map