'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _buffer = require('buffer');

var _pako = require('pako');

var _pako2 = _interopRequireDefault(_pako);

var _shasum = require('shasum');

var _shasum2 = _interopRequireDefault(_shasum);

var _read = require('../utils/read');

var _read2 = _interopRequireDefault(_read);

var _write = require('../utils/write');

var _write2 = _interopRequireDefault(_write);

var _exists = require('../utils/exists');

var _exists2 = _interopRequireDefault(_exists);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function wrapObject({ type, object /*: {type: string, object: Buffer} */ }) {
  let buffer = _buffer.Buffer.concat([_buffer.Buffer.from(type + ' '), _buffer.Buffer.from(object.byteLength.toString()), _buffer.Buffer.from([0]), _buffer.Buffer.from(object)]);
  let oid = (0, _shasum2.default)(buffer);
  return {
    oid,
    file: _buffer.Buffer.from(_pako2.default.deflate(buffer))
  };
}

function unwrapObject({ oid, file /*: {oid: string, file: Buffer} */ }) {
  let inflated = _buffer.Buffer.from(_pako2.default.inflate(file));
  if (oid) {
    let sha = (0, _shasum2.default)(inflated);
    if (sha !== oid) {
      throw new Error(`SHA check failed! Expected ${oid}, computed ${sha}`);
    }
  }
  let s = inflated.indexOf(32); // first space
  let i = inflated.indexOf(0); // first null value
  let type = inflated.slice(0, s).toString('utf8'); // get type of object
  let length = inflated.slice(s + 1, i).toString('utf8'); // get type of object
  let actualLength = inflated.length - (i + 1);
  // verify length
  if (parseInt(length) !== actualLength) {
    throw new Error(`Length mismatch: expected ${length} bytes but got ${actualLength} instead.`);
  }
  return {
    type,
    object: _buffer.Buffer.from(inflated.slice(i + 1))
  };
}

class GitObjectManager {
  static async read({ gitdir, oid /*: {gitdir: string, oid: string} */ }) {
    let file = await (0, _read2.default)(`${gitdir}/objects/${oid.slice(0, 2)}/${oid.slice(2)}`);
    if (!file) throw new Error(`Git object with oid ${oid} not found`);
    let { type, object } = unwrapObject({ oid, file });
    return { type, object };
  }

  static async write({ gitdir, type, object }) /*: Promise<string> */{
    let { file, oid } = wrapObject({ type, object });
    let filepath = `${gitdir}/objects/${oid.slice(0, 2)}/${oid.slice(2)}`;
    // Don't overwrite existing git objects - this helps avoid EPERM errors.
    // Although I don't know how we'd fix corrupted objects then. Perhaps delete them
    // on read?
    if (!(await (0, _exists2.default)(filepath))) await (0, _write2.default)(filepath, file);
    return oid;
  } /*: {gitdir: string, type: string, object: Buffer} */
}
exports.default = GitObjectManager;
//# sourceMappingURL=GitObjectManager.js.map