'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _buffer = require('buffer');

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _pktLineReader = require('../utils/pkt-line-reader');

var _pktLineReader2 = _interopRequireDefault(_pktLineReader);

var _simpleGet = require('simple-get');

var _simpleGet2 = _interopRequireDefault(_simpleGet);

var _pify = require('pify');

var _pify2 = _interopRequireDefault(_pify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function basicAuth(auth) {
  return `Basic ${_buffer.Buffer.from(auth.username + ':' + auth.password).toString('base64')}`;
}

class GitRemoteHTTP {
  /*::
  GIT_URL : string
  refs : Map<string, string>
  capabilities : Set<string>
  auth : { username : string, password : string }
  */
  constructor(url /*: string */) {
    // Auto-append the (necessary) .git if it's missing.
    if (!url.endsWith('.git')) url = url += '.git';
    this.GIT_URL = url;
  }
  async preparePull() {
    await this.discover('git-upload-pack');
  }
  async preparePush() {
    await this.discover('git-receive-pack');
  }
  async discover(service /*: string */) {
    this.capabilities = new Set();
    this.refs = new Map();
    let res = await _axios2.default.get(`${this.GIT_URL}/info/refs?service=${service}`, {
      auth: this.auth
    });
    (0, _assert2.default)(res.status === 200, `Bad status code from server: ${res.status}`);
    // There is probably a better way to do this, but for now
    // let's just throw the result parser inline here.
    let read = new _pktLineReader2.default(res.data);
    let lineOne = read();
    // skip past any flushes
    while (lineOne === null) lineOne = read();
    (0, _assert2.default)(lineOne !== true, 'Bad response from git server.');
    (0, _assert2.default)(lineOne.toString('utf8') === `# service=${service}\n`, lineOne.toString('utf8'));
    let lineTwo = read();
    // skip past any flushes
    while (lineTwo === null) lineTwo = read();
    // In the edge case of a brand new repo, zero refs (and zero capabilities)
    // are returned.
    if (lineTwo === true) return;
    let [firstRef, capabilities] = lineTwo.toString('utf8').trim().split('\0');
    capabilities.split(' ').map(x => this.capabilities.add(x));
    let [ref, name] = firstRef.split(' ');
    this.refs.set(name, ref);
    while (true) {
      let line = read();
      if (line === true) break;
      if (line !== null) {
        let [ref, name] = line.toString('utf8').trim().split(' ');
        this.refs.set(name, ref);
      }
    }
  }
  async push(stream) {
    const service = 'git-receive-pack';
    let headers = {};
    headers['Content-Type'] = `application/x-${service}-request`;
    headers['Accept'] = `application/x-${service}-result`;
    if (this.auth) {
      headers['Authorization'] = basicAuth(this.auth);
    }
    let res = await (0, _pify2.default)(_simpleGet2.default)({
      method: 'POST',
      url: `${this.GIT_URL}/${service}`,
      body: stream,
      headers
    });
    return res;
  }
  async pull({ stream, refs }) {
    const service = 'git-upload-pack';
    let res = await _axios2.default.post(`${this.GIT_URL}/${service}`, {
      data: stream,
      headers: {
        'Content-Type': `application/x-${service}-request`,
        Accept: `application/x-${service}-result`
      },
      auth: this.auth
    });
    (0, _assert2.default)(res.status === 200);
    return res;
  }
}
exports.default = GitRemoteHTTP;
//# sourceMappingURL=GitRemoteHTTP.js.map